@@ -34,6 +34,7 @@ int		enable_socket = 0;
 int		single_instance = 0; /* only allow one xxxterm to run */
 int		fancy_bar = 1;	/* fancy toolbar */
 int		browser_mode = XT_BM_NORMAL;
+int		gui_mode = XT_GM_CLASSIC;
 int		enable_localstorage = 1;
 char		*statusbar_elems = NULL;
 
@@ -51,6 +52,7 @@ gfloat		default_zoom_level = 1.0;
 char		default_script[PATH_MAX];
 int		window_height = 768;
 int		window_width = 1024;
+int		window_maximize = 0;
 int		icon_size = 2; /* 1 = smallest, 2+ = bigger */
 int		refresh_interval = 10; /* download refresh interval */
 int		enable_plugin_whitelist = 0;
@@ -68,8 +70,9 @@ char		*http_proxy = NULL;
 char		download_dir[PATH_MAX];
 char		runtime_settings[PATH_MAX]; /* override of settings */
 int		allow_volatile_cookies = 0;
+int		color_visited_uris = 1;
 int		save_global_history = 0; /* save global history to disk */
-char		*user_agent = NULL;
+struct user_agent	*user_agent = NULL;
 int		save_rejected_cookies = 0;
 int		session_autosave = 0;
 int		guess_search = 0;
@@ -87,6 +90,12 @@ char		*encoding = NULL;
 int		autofocus_onload = 0;
 int		js_autorun_enabled = 1;
 int		edit_mode = XT_EM_HYBRID;
+int		userstyle_global = 0;
+int		auto_load_images = 1;
+int		enable_autoscroll = 0;
+int		enable_favicon_entry = 1;
+int		enable_favicon_tabs = 0;
+char		*external_editor = NULL;
 
 char		*cmd_font_name = NULL;
 char		*oops_font_name = NULL;
@@ -106,6 +115,7 @@ int		add_pl_wl(struct settings *, char *);
 int		add_mime_type(struct settings *, char *);
 int		add_alias(struct settings *, char *);
 int		add_kb(struct settings *, char *);
+int		add_ua(struct settings *, char *);
 
 int		set_download_dir(struct settings *, char *);
 int		set_default_script(struct settings *, char *);
@@ -113,6 +123,11 @@ int		set_runtime_dir(struct settings *, char *);
 int		set_tab_style(struct settings *, char *);
 int		set_edit_mode(struct settings *, char *);
 int		set_work_dir(struct settings *, char *);
+int		set_auto_load_images(char *value);
+int		set_enable_autoscroll(char *value);
+int		set_enable_favicon_entry(char *value);
+int		set_enable_favicon_tabs(char *value);
+int		set_external_editor(char *);
 
 void		walk_mime_type(struct settings *, void (*)(struct settings *,
 		    char *, void *), void *);
@@ -126,6 +141,8 @@ void		walk_pl_wl(struct settings *, void (*)(struct settings *,
 		    char *, void *), void *);
 void		walk_kb(struct settings *, void (*)(struct settings *, char *,
 		    void *), void *);
+void		walk_ua(struct settings *, void (*)(struct settings *, char *,
+		    void *), void *);
 
 int
 set_http_proxy(char *proxy)
@@ -166,6 +183,12 @@ struct special		s_browser_mode = {
 	NULL
 };
 
+struct special		s_gui_mode = {
+	set_gui_mode,
+	get_gui_mode,
+	NULL
+};
+
 struct special		s_cookie = {
 	set_cookie_policy,
 	get_cookie_policy,
@@ -238,11 +261,19 @@ struct special		s_edit_mode = {
 	NULL
 };
 
+struct special		s_ua = {
+	add_ua,
+	NULL,
+	walk_ua
+};
+
 struct settings		rs[] = {
 	{ "allow_volatile_cookies",	XT_S_INT, 0,		&allow_volatile_cookies, NULL, NULL },
 	{ "append_next",		XT_S_INT, 0,		&append_next, NULL, NULL },
 	{ "autofocus_onload",		XT_S_INT, 0,		&autofocus_onload, NULL, NULL },
 	{ "browser_mode",		XT_S_INT, 0, NULL, NULL,&s_browser_mode },
+	{ "gui_mode",			XT_S_INT, 0, NULL, NULL,&s_gui_mode },
+	{ "color_visited_uris",		XT_S_INT, 0,		&color_visited_uris, NULL, NULL },
 	{ "cookie_policy",		XT_S_INT, 0, NULL, NULL,&s_cookie },
 	{ "cookies_enabled",		XT_S_INT, 0,		&cookies_enabled, NULL, NULL },
 	{ "ctrl_click_focus",		XT_S_INT, 0,		&ctrl_click_focus, NULL, NULL },
@@ -259,6 +290,7 @@ struct settings		rs[] = {
 	{ "enable_socket",		XT_S_INT, XT_SF_RESTART,&enable_socket, NULL, NULL },
 	{ "enable_spell_checking",	XT_S_INT, 0,		&enable_spell_checking, NULL, NULL },
 	{ "encoding",			XT_S_STR, 0, NULL,	&encoding, NULL },
+	{ "external_editor",		XT_S_STR,0, NULL,	&external_editor, NULL, NULL, set_external_editor },
 	{ "fancy_bar",			XT_S_INT, XT_SF_RESTART,&fancy_bar, NULL, NULL },
 	{ "guess_search",		XT_S_INT, 0,		&guess_search, NULL, NULL },
 	{ "history_autosave",		XT_S_INT, 0,		&history_autosave, NULL, NULL },
@@ -286,11 +318,15 @@ struct settings		rs[] = {
 	{ "statusbar_elems",		XT_S_STR, 0, NULL,	&statusbar_elems, NULL },
 	{ "tab_style",			XT_S_STR, 0, NULL, NULL,&s_tab_style },
 	{ "url_regex",			XT_S_STR, 0, NULL,	&url_regex, NULL },
-	{ "user_agent",			XT_S_STR, 0, NULL,	&user_agent, NULL },
 	{ "window_height",		XT_S_INT, 0,		&window_height, NULL, NULL },
 	{ "window_width",		XT_S_INT, 0,		&window_width, NULL, NULL },
+	{ "window_maximize",		XT_S_INT, 0,		&window_maximize, NULL, NULL },
 	{ "work_dir",			XT_S_STR, 0, NULL, NULL,&s_work_dir },
 	{ "xterm_workaround",		XT_S_INT, 0,		&xterm_workaround, NULL, NULL },
+	{ "auto_load_images",		XT_S_INT, 0, 		&auto_load_images, NULL, NULL, NULL, set_auto_load_images },
+	{ "enable_autoscroll",		XT_S_INT, 0, 		&enable_autoscroll, NULL, NULL, NULL, set_enable_autoscroll },
+	{ "enable_favicon_entry",	XT_S_INT, 0, 		&enable_favicon_entry, NULL, NULL, NULL, set_enable_favicon_entry },
+	{ "enable_favicon_tabs",	XT_S_INT, 0, 		&enable_favicon_tabs, NULL, NULL, NULL, set_enable_favicon_tabs },
 
 	/* font settings */
 	{ "cmd_font",			XT_S_STR, 0, NULL, &cmd_font_name, NULL },
@@ -305,6 +341,7 @@ struct settings		rs[] = {
 	{ "keybinding",			XT_S_STR, XT_SF_RUNTIME, NULL, NULL, &s_kb },
 	{ "mime_type",			XT_S_STR, XT_SF_RUNTIME, NULL, NULL, &s_mime },
 	{ "pl_wl",			XT_S_STR, XT_SF_RUNTIME, NULL, NULL, &s_pl },
+	{ "user_agent",			XT_S_STR, XT_SF_RUNTIME, NULL,	NULL, &s_ua },
 };
 
 size_t
@@ -433,6 +470,42 @@ get_browser_mode(struct settings *s)
 	return (r);
 }
 
+int
+set_gui_mode(struct settings *s, char *val)
+{
+	if (!strcmp(val, "classic")) {
+		fancy_bar = 1;
+		show_tabs = 1;
+		tab_style = XT_TABS_NORMAL;
+		show_url = 1;
+		show_statusbar = 0;
+	} else if (!strcmp(val, "minimal")) {
+		fancy_bar = 0;
+		show_tabs = 1;
+		tab_style = XT_TABS_COMPACT;
+		show_url = 0;
+		show_statusbar = 1;
+	} else
+		return (1);
+
+	return (0);
+}
+
+char *
+get_gui_mode(struct settings *s)
+{
+	char			*r = NULL;
+
+	if (gui_mode == XT_GM_CLASSIC)
+		r = g_strdup("classic");
+	else if (browser_mode == XT_GM_MINIMAL)
+		r = g_strdup("minimal");
+	else
+		return (NULL);
+
+	return (r);
+}
+
 int
 set_cookie_policy(struct settings *s, char *val)
 {
@@ -662,6 +735,7 @@ struct key_binding	keys[] = {
 	{ "help",		0,	1,	GDK_F1		},
 	{ "run_script",		MOD1,	1,	GDK_r		},
 	{ "proxy toggle",	0,	1,	GDK_F2		},
+	{ "editelement",	CTRL,	1,	GDK_i		},
 
 	/* search */
 	{ "searchnext",		0,	0,	GDK_n		},
@@ -679,6 +753,7 @@ struct key_binding	keys[] = {
 
 	/* custom stylesheet */
 	{ "userstyle",		0,	0,	GDK_s		},
+	{ "userstyle_global",	SHFT,	0,	GDK_S		},
 
 	/* navigation */
 	{ "goback",		0,	0,	GDK_BackSpace	},
@@ -920,6 +995,86 @@ add_kb(struct settings *s, char *entry)
 	return (keybinding_add(entry, key, key[0] == '!'));
 }
 
+int
+add_ua(struct settings *s, char *value)
+{
+	struct user_agent *ua;
+
+	ua = g_malloc0(sizeof *ua);
+	ua->value = g_strdup(value);
+
+	TAILQ_INSERT_HEAD(&ua_list, ua, entry);
+
+	/* use the last added user agent */
+	user_agent = TAILQ_FIRST(&ua_list);
+	user_agent_count++;
+
+	return (0);
+}
+
+
+void
+walk_ua(struct settings *s,
+    void (*cb)(struct settings *, char *, void *), void *cb_args)
+{
+	struct user_agent		*ua;
+
+	if (s == NULL || cb == NULL) {
+		show_oops(NULL, "walk_ua invalid parameters");
+		return;
+	}
+
+	TAILQ_FOREACH(ua, &ua_list, entry) {
+		cb(s, ua->value, cb_args);
+	}
+}
+
+int
+set_auto_load_images(char *value)
+{
+	struct tab *t;
+
+	auto_load_images = atoi(value);
+	TAILQ_FOREACH(t, &tabs, entry) {
+		g_object_set(G_OBJECT(t->settings),
+		    "auto-load-images", auto_load_images, (char *)NULL);
+		webkit_web_view_set_settings(t->wv, t->settings);
+	}
+	return (0);
+}
+
+int
+set_enable_autoscroll(char *value)
+{
+	enable_autoscroll = atoi(value);
+	return (0);
+}
+
+int
+set_enable_favicon_entry(char *value)
+{
+	enable_favicon_entry = atoi(value);
+	return (0);
+}
+
+int
+set_enable_favicon_tabs(char *value)
+{
+	enable_favicon_tabs = atoi(value);
+	return (0);
+}
+
+int
+set_external_editor(char *editor)
+{
+	if (external_editor)
+		g_free(external_editor);
+
+	external_editor = g_strdup(editor);
+
+	return (0);
+}
+
 void
 setup_proxy(char *uri)
 {
